import stylelint from 'stylelint';

const physicalShorthandLogicalPropertiesMap = [[['top', 'left', 'bottom', 'right'], 'inset'], [['margin-top', 'margin-left', 'margin-bottom', 'margin-right'], 'margin'], [['padding-top', 'padding-left', 'padding-bottom', 'padding-right'], 'padding']];
const physicalShorthandPropertiesMap = [[['top', 'bottom'], 'inset-block'], [['left', 'right'], 'inset-inline'], [['top', 'left'], 'inset-start'], [['bottom', 'right'], 'inset-end'], [['margin-top', 'margin-bottom'], 'margin-block'], [['margin-left', 'margin-right'], 'margin-inline'], [['margin-top', 'margin-left'], 'margin-start'], [['margin-bottom', 'margin-right'], 'margin-end'], [['padding-top', 'padding-bottom'], 'padding-block'], [['padding-left', 'padding-right'], 'padding-inline'], [['padding-top', 'padding-left'], 'padding-start'], [['padding-bottom', 'padding-right'], 'padding-end']];
const physicalPropertiesMap = [[['top'], 'inset-block-start'], [['right'], 'inset-inline-end'], [['bottom'], 'inset-block-end'], [['left'], 'inset-inline-start'], [['margin-top'], 'margin-block-start'], [['margin-right'], 'margin-inline-end'], [['margin-bottom'], 'margin-block-end'], [['margin-left'], 'margin-inline-start'], [['padding-top'], 'padding-block-start'], [['padding-right'], 'padding-inline-end'], [['padding-bottom'], 'padding-block-end'], [['padding-left'], 'padding-inline-start']];
const physicalPropertyValuesMap = [[/^clear$/i, {
  left: 'inline-start',
  right: 'inline-end'
}], [/^float$/i, {
  left: 'inline-start',
  right: 'inline-end'
}], [/^text-align$/i, {
  left: 'start',
  right: 'end'
}]];

const validateRuleWithProps = (root, props, fn) => {
  // conditionally walk nodes with children
  if (root.nodes && root.nodes.length) {
    const args = [];
    const hasProps = props.every(prop => {
      const declIndex = root.nodes.findIndex(child => child.type === 'decl' && child.prop === prop);
      const decl = root.nodes[declIndex];

      if (decl) {
        args.push(decl, declIndex);
      }

      return decl;
    });

    if (hasProps) {
      fn(...args);
    }
  }
};

var ruleName = 'csstools/stylelint-use-logical';

var messages = stylelint.utils.ruleMessages(ruleName, {
  unexpectedProp(property, logicalProperty) {
    return `Unexpected ${property} property. Use ${logicalProperty}.`;
  },

  unexpectedValue(property, value, logicalValue) {
    return `Unexpected "${value}" value in ${property} property. Use "${logicalValue}".`;
  }

});

// walk all container nodes
function walk(node, fn) {
  if (node.nodes && node.nodes.length) {
    const nodes = node.nodes.slice();
    const length = nodes.length;
    let index = -1;

    while (++index < length) {
      const child = nodes[index];
      fn(child);
      walk(child, fn);
    }
  }
}

const reportedDecls = new WeakMap();
var index = stylelint.createPlugin(ruleName, (method, opts, context) => {
  const propExceptions = [].concat(Object(opts).except || []);
  const isAutofix = isContextAutofixing(context);
  return (root, result) => {
    // validate the method
    const isMethodValid = stylelint.utils.validateOptions(result, ruleName, {
      actual: method,

      possible() {
        return isMethodIndifferent(method) || isMethodAlways(method);
      }

    });

    if (isMethodValid && isMethodAlways(method)) {
      walk(root, node => {
        // validate or autofix multiple physical properties
        physicalShorthandLogicalPropertiesMap.forEach(([props, prop]) => {
          validateRuleWithProps(node, props, (blockStartDecl, blockStartIndex, inlineStartDecl, inlineStartIndex, blockEndDecl, blockEndIndex, inlineEndDecl, inlineEndIndex) => {
            // eslint-disable-line
            const firstInlineDecl = blockStartDecl;

            if (isAutofix) {
              const values = [blockStartDecl.value, inlineStartDecl.value, blockEndDecl.value, inlineEndDecl.value];

              if (values[1] === values[3]) {
                values.pop();

                if (values[2] === values[1]) {
                  values.pop();

                  if (values[1] === values[0]) {
                    values.pop();
                  }
                }
              }

              firstInlineDecl.cloneBefore({
                prop,
                value: values.length <= 2 ? values.join(' ') : `logical ${values.join(' ')}`
              });
              blockStartDecl.remove();
              inlineStartDecl.remove();
              blockEndDecl.remove();
              inlineEndDecl.remove();
            } else if (!isDeclReported(blockStartDecl) && !isDeclReported(inlineStartDecl) && !isDeclReported(blockEndDecl) && !isDeclReported(inlineEndDecl)) {
              stylelint.utils.report({
                message: messages.unexpectedProp(firstInlineDecl.prop, prop),
                node: firstInlineDecl,
                result,
                ruleName
              });
              reportedDecls.set(blockStartDecl);
              reportedDecls.set(inlineStartDecl);
              reportedDecls.set(blockEndDecl);
              reportedDecls.set(inlineEndDecl);
            }
          });
        }); // validate physical shorthand properties

        physicalShorthandPropertiesMap.forEach(([props, prop]) => {
          validateRuleWithProps(node, props, (blockStartDecl, blockStartIndex, inlineStartDecl, inlineStartIndex) => {
            // eslint-disable-line
            const firstInlineDecl = blockStartIndex < inlineStartIndex ? blockStartDecl : inlineStartDecl;

            if (isAutofix) {
              firstInlineDecl.cloneBefore({
                prop,
                value: blockStartDecl.value === inlineStartDecl.value ? blockStartDecl.value : [blockStartDecl.value, inlineStartDecl.value].join(' ')
              });
              blockStartDecl.remove();
              inlineStartDecl.remove();
            } else if (!isDeclReported(blockStartDecl) && !isDeclReported(inlineStartDecl)) {
              stylelint.utils.report({
                message: messages.unexpectedProp(firstInlineDecl.prop, prop),
                node: firstInlineDecl,
                result,
                ruleName
              });
              reportedDecls.set(blockStartDecl);
              reportedDecls.set(inlineStartDecl);
            }
          });
        }); // validate or autofix physical properties (like left, margin-right, padding-bottom)

        physicalPropertiesMap.forEach(([props, prop]) => {
          validateRuleWithProps(node, props, physicalDecl => {
            if (!isDeclAnException(physicalDecl, propExceptions)) {
              if (isAutofix) {
                physicalDecl.prop = prop;
              } else if (!isDeclReported(physicalDecl)) {
                stylelint.utils.report({
                  message: messages.unexpectedProp(physicalDecl.prop, prop),
                  node: physicalDecl,
                  result,
                  ruleName
                });
                reportedDecls.set(physicalDecl);
              }
            }
          });
        }); // validate physical property values

        physicalPropertyValuesMap.forEach(([regexp, props]) => {
          if (node.type === 'decl' && regexp.test(node.prop) && !isDeclAnException(node, propExceptions)) {
            const valuekey = node.value.toLowerCase();

            if (valuekey in props) {
              const value = props[valuekey];

              if (isAutofix) {
                node.value = value;
              } else {
                stylelint.utils.report({
                  message: messages.unexpectedValue(node.prop, node.value, value),
                  node,
                  result,
                  ruleName
                });
                reportedDecls.set(node);
              }
            }
          }
        });
      });
    }
  };
});

const isMethodIndifferent = method => method === 'ignore' || method === null || method === false;

const isMethodAlways = method => method === 'always' || method === true;

const isContextAutofixing = context => Boolean(Object(context).fix);

const isDeclAnException = (decl, propExceptions) => propExceptions.some(match => match instanceof RegExp ? match.test(decl.prop) : String(match || '').toLowerCase() === String(decl.prop || '').toLowerCase());

const isDeclReported = decl => reportedDecls.has(decl);

export default index;
export { ruleName };
//# sourceMappingURL=index.es.mjs.map
